<?php declare(strict_types=1);

namespace Infomaker\Everyware\Base\OpenContent;

use Infomaker\Everyware\Base\ContentProvider;
use Infomaker\Everyware\Base\PropertyMapConfig;
use Infomaker\Everyware\Base\PropertyParser;
use Infomaker\Everyware\Base\PropertyMapper;
use OcAPI;

class OpenContentProvider implements ContentProvider
{

    /**
     * @var PropertyMapper
     */
    protected static $propertyMapper;

    /**
     * @var PropertyParser
     */
    protected static $propertyParser;

    /**
     * @var string
     */
    private $contentReference;

    /**
     * @var OcAPI
     */
    private $oc;

    /**
     * @var string
     */
    private $queryParams;

    /**
     * @var bool
     */
    private $useCache;

    /**
     * @var array
     */
    private static $mappedProperties = [];

    public function __construct(OcAPI $oc, array $queryParams = [], $useCache = true)
    {
        $this->oc = $oc;
        $this->queryParams = $queryParams;
        $this->useCache = $useCache;
    }

    public static function setupPropertyMapper(PropertyMapConfig $config): void
    {
        static::$propertyMapper = new PropertyMapper($config);
        static::$propertyParser = new PropertyParser($config);
    }

    public static function setupDefaultSearchProperties(array $properties = []): void
    {
        \add_filter('ew_apply_default_search_properties', function ($filteredProperties) use ($properties) {
            return array_merge($filteredProperties, $properties);
        });
    }

    public static function setupNotifierProperties($contenttype, array $properties = []): void
    {
        if (empty($properties) && static::$propertyMapper) {
            $properties = static::$propertyMapper->requiredProperties($contenttype);
        }

        $contenttypeVar = strtolower($contenttype);

        add_filter("ew_notifier_update_{$contenttypeVar}_properties",
            function ($filteredProperties) use ($properties) {
                return array_merge($filteredProperties, $properties);
            });
    }

    public static function setupRequiredSearchProperties(iterable $requiredProperties): void
    {
        add_filter('ew_apply_required_search_properties', function ($properties) use ($requiredProperties) {

            $searchProperties = [$properties];

            foreach ($requiredProperties as $trigger => $propertiesToAdd) {
                if (\in_array($trigger, $properties, true)) {
                    $searchProperties[] = (array)$propertiesToAdd;
                }
            }

            return \array_unique(\array_merge(...$searchProperties));
        });
    }

    public function queryWithRequirements(array $requirements = []): array
    {
        if ($this->contentReference) {
            $requirements['properties'] = $this->mergeMappedProperties($this->contentReference,
                $requirements['properties'] ?? []);

            return $this->mapResult($this->getContent($requirements));
        }

        return $this->getContent($requirements);
    }


    public function setPropertyMap(string $reference): void
    {
        $this->contentReference = $reference;
    }

    private function mapResult(array $result = []): array
    {
        return static::$propertyParser->fromSearch($this->contentReference, $result)->all();
    }

    private function cleanResult(array $result = []): array
    {
        unset($result['facet'], $result['hits'], $result['duration']);

        return $result;
    }

    public static function setup(array $queryParams = [], $useCache = true)
    {
        return new static(new OcAPI(), $queryParams, $useCache);
    }

    public function mergeMappedProperties($contentReference, array $properties = []): array
    {
        return array_unique(array_merge($properties, $this->getPropertiesByReference($contentReference)));
    }

    private function getContent(array $requirements): array
    {
        return $this->cleanResult($this->oc->search(array_replace($this->queryParams, $requirements), $this->useCache));
    }

    public function getPropertiesByReference(string $reference = ''): array
    {
        if ( ! array_key_exists($reference, static::$mappedProperties)) {
            static::$mappedProperties[$reference] = static::$propertyMapper->requiredProperties($reference);
        }

        return static::$mappedProperties[$reference];
    }

    public function getObject(string $uuid, string $propertyReference = '')
    {
        if ($this->contentReference) {
            $result = $this->oc->get_single_object(
                $uuid,
                $this->getPropertiesByReference($this->contentReference)
            );

            return static::$propertyParser->fromOcObject($this->contentReference, $result);
        }

        return $this->oc->get_single_object(
            $uuid,
            $this->getPropertiesByReference($propertyReference)
        );
    }
}
